/* -
 * (C) NTT TechnoCross Corporation 2019 All Rights Reserved
 */

#include <stdio.h>

#include <rte_ethdev.h> /**<パケット処理API */
#include <rte_mbuf.h>   /**<パケットバッファAPI */

#include "fwd.h"
#include "log.h"

/**
 *  [デバック用]パケット先頭64byte出力処理
 *  - コンパイルオプションで"APP_DEBUG"を有効にした場合にのみ
 *    このコードをコンパイルする
 */
#ifdef APP_DEBUG
static void _dump_pkt_head_64byte(struct rte_mbuf *pkt) {
  // 6byteの情報を16進数のchar型で確保する配列を用意。
  // 1byteの情報は16進数では2文字で表現できるため、6*2+1(終端文字)byteの領域を確保する。
  char str[(6 * 2) + 1] = {0};

  // パケットバッファのアドレスを16進文字に変換して領域に書き込みログに出力する
  uint8_t *dat = rte_pktmbuf_mtod(pkt, uint8_t *);
  for (int i = 0; i < 6; i++) sprintf(&str[i * 2], "%02x", *(dat + i));
  RTE_LOG_DP(DEBUG, APP_FWD, "[Dump 6byte] %s\n", str);
}
#endif

/**
 *  パケット転送処理(メインループ)
 *  - 指定のportから受信したパケットを同じportに転送する
 */
void fwd_main_loop(uint16_t port_id) {
  struct rte_mbuf *pkts[32];    /* パケット取得用の領域 */
  uint16_t nb_rx, nb_tx, i = 0; /* パケット受信/送信数 */
  RTE_LOG_DP(INFO, APP_FWD,
             "Forwards packets received on port %u to port %u.\n", port_id,
             port_id);
  /**
   *  - パケットを受信したら、受信したパケットを同じportに転送
   *  - もしパケット送信に失敗したら送信しようとしたパケットを破棄
   */
  while (1) {
    nb_rx = rte_eth_rx_burst(port_id, 0, pkts, 32);
    if (nb_rx <= 0) continue;
#ifdef APP_DEBUG
    /**
     * [デバック用]パケット先頭6byte出力処理
     *  - コンパイルオプションで"APP_DEBUG"を有効にした場合にのみ
     *    このコードをコンパイルする
     */
    for (i = 0; i < nb_rx; i++) _dump_pkt_head_64byte(pkts[i]);
#endif
    nb_tx = rte_eth_tx_burst(port_id, 0, pkts, nb_rx);
    if (nb_tx < nb_rx)
      for (i = nb_tx; i < nb_rx; i++) rte_pktmbuf_free(pkts[i]);
  }
}
