/* -
 * (C) NTT TechnoCross Corporation 2019 All Rights Reserved
 */

#include "log.h"

/**
 *  ログのグローバルレベル
 *  - コンパイルオプションで"APP_DEBUG"を有効にした場合、
 *    グローバルのログレベルを「RTE_LOG_DEBUG」に設定するようにしている
 */
#ifdef APP_DEBUG
#define APP_G_LOGLEVEL RTE_LOG_DEBUG
#else
#define APP_G_LOGLEVEL RTE_LOG_INFO
#endif

/**
 *  ログ種別格納用のグローバル変数
 *  - 外部ソースコードからextern出来るようにstaticにしない
 */
int logtype_id_main;
int logtype_id_fwd;

/**
 *  ログ初期化
 *  - ログ種別を設定し、グローバル変数にログ種別のIDを登録
 *  - ログのグローバルレベルを設定
 *  - 各アプリケーション用のログレベルを設定
 */
void log_initialize(void) {
  logtype_id_main = rte_log_register("app_main");
  logtype_id_fwd = rte_log_register("app_fwd");
  rte_log_set_global_level(APP_G_LOGLEVEL);
  rte_log_set_level(RTE_LOGTYPE_APP_MAIN, RTE_LOG_DEBUG);
  rte_log_set_level(RTE_LOGTYPE_APP_FWD, RTE_LOG_DEBUG);
}
